import glob
import os
import re
import sys

import pytest

current_script = os.path.abspath(__file__)
base_dir = os.path.dirname(current_script)
sys.path.insert(0, os.path.abspath(f"{base_dir}/../tools"))

from parse_fwu import FWUFile

fwu_files = glob.glob("fw_*_*.fwu")


@pytest.mark.parametrize("fwu_file", fwu_files)
def test_filename(fwu_file: str) -> None:
    fwu = FWUFile(fwu_file)

    assert re.match(
        r"fw_\d{5}_\d{3}.fwu", fwu_file
    ), "Filename does not match naming convention"

    _, _module_type, _hardware_revision = fwu_file[:-4].split("_")

    module_type = int(_module_type)
    hardware_revision = int(_hardware_revision)

    assert module_type == fwu.module_type, (
        f"Module type in filename ({module_type}) does not match "
        + f"fwu content ({fwu.module_type})"
    )

    assert hardware_revision == fwu.hardware_revision, (
        f"Hardware revision in filename ({hardware_revision}) does not match "
        + f"fwu content ({fwu.hardware_revision})"
    )


@pytest.mark.parametrize("fwu_file", fwu_files)
def test_fwu_content(fwu_file: str) -> None:
    fwu = FWUFile(fwu_file)

    assert (
        fwu.version_major >= 1
    ), "Major version of a released firmware must be 1 or greater"
    assert fwu.version_minor >= 0, "Minor version must be 0 or greater"
    assert fwu.hardware_revision >= 1, "Hardware revision must be 1 or greater"

    assert fwu.svn_tag or fwu.git_hash, "Either a SVN tag or a GIT hash must be present"

    assert fwu.fwu_revision in (1, 3), "Invalid FWU revision"
